import cv2
import numpy as np
import socket
import struct

UDP_PACKET_SIZE = 65507
PORT = 35100
ADDRESS = '192.168.7.1'

# Return the packet number
def getPacketNum(data):
    return struct.unpack("B", data[0:1])[0]

# Wait for a new frame to be sent over the socket before interpreting data
def waitForNewFrame(socket):
    while True:
        packet = socket.recvfrom(UDP_PACKET_SIZE)[0]
        if getPacketNum(packet) == 1:
            break


def main():
    # Initialize the socket for UDP
    receiverSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    receiverSocket.bind((ADDRESS, PORT))

    # Empty the bytes of the frame and wait for any previous frame to finish
    frame = b''
    waitForNewFrame(receiverSocket)
    isRunning = True

    
    while isRunning:
        # Receive a pack over the UDP socket
        packet = receiverSocket.recvfrom(UDP_PACKET_SIZE)[0]

        # Add on the packet onto the other packets to form a frame
        frame += packet[1:]

        # On the last packet, show the frame
        if getPacketNum(packet) == 1:
            isRunning = showImage(frame)
            frame = b''

    cv2.destroyAllWindows()
    receiverSocket.close()
    print("Exiting webcam Viewer")

def showImage(frame):
    # Decode the frame into an image
    image = cv2.imdecode(np.fromstring(frame, dtype=np.uint8), 1)
    # Display the image
    cv2.imshow('webcam',image)

    # If 'q' is press on the keyboard, exit the program
    if cv2.waitKey(1) & 0xFF == ord('q'):
        return False
    else:
        return True
    

if __name__ == "__main__":
    main()
