import cv2
import socket
import math
import struct

# max payload for IPv4 UDP packet in bytes
UDP_PACKET_SIZE = 65507
#we use first byte of data to store the number of samples
FRAME_DATA_SIZE = UDP_PACKET_SIZE -1
PORT = 35100
ADDRESS = "192.168.7.1"

def encodeAndSendImage(socket, image):

    #Encode the image as a jpeg
    encodedImage = cv2.imencode('.jpg', image)[1]
    #convert the encoded image to a string
    encodedData = encodedImage.tostring()

    #Find the size of the data to determine the number of UDP packets
    sizeOfData = len(encodedData)
    numPackets = math.ceil(sizeOfData / FRAME_DATA_SIZE)
    
    #Send UDP packets over the socket with the first byte of the data being the packet number
    startIndex = 0
    for numPackets in range (numPackets, 0, -1):
        endIndex = startIndex + FRAME_DATA_SIZE
        socket.sendto(struct.pack("B", numPackets) + encodedData[startIndex:endIndex], (ADDRESS, PORT))
        startIndex = endIndex


def main():
    # initialize the socket (UDP)
    senderSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    # Capture the camera image and send it over the socket
    capturedImage = cv2.VideoCapture(0)
    while (capturedImage.isOpened()):
        imageRead = capturedImage.read()[1]
        encodeAndSendImage(senderSocket,imageRead)

    # Stop sending packets
    capturedImage.release()
    cv2.destroyAllWindows()
    socket.close()

if __name__ == "__main__":
    main()